//=============================================================================
// WoEHitEffect - Handles spawning of different effects on different surfaces.
// http://come.to/MrEvil
//=============================================================================
class WoEHitEffect extends Actor;

#exec OBJ LOAD FILE=WoE_Effects_Textures.utx
#exec OBJ LOAD FILE=VMParticleTextures.utx
#exec OBJ LOAD FILE=VehicleFX.utx
#exec OBJ LOAD FILE=XGameShadersB.utx
#exec OBJ LOAD FILE=EmitterTextures.utx

var() class<WoEHitEmitter> HitEffectClasses[11]; //Effects indexed by surface type.
var() class<WoEBulletDecal> DecalClasses[11];

static function StaticPrecache(LevelInfo L)
{
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.pock0_t');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.pock2_t');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.pock4_t');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.GlassMark1');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.GlassMark2');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.GlassMark3');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.MetalMark1');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.MetalMark2');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.MetalMark3');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.SnowMark1');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.SnowMark2');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.SnowMark3');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.WoodMark1');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.WoodMark2');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.WoodMark3');

	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.GlassChips');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.WoodChips');
	L.AddPrecacheMaterial(Texture'WoE_Effects_Textures.PlantBits');
	L.AddPrecacheMaterial(Texture'VMParticleTextures.DirtKICKGROUP.dirtKICKTEX');
	L.AddPrecacheMaterial(Texture'VMParticleTextures.DirtKICKGROUP.snowKICKTEX');
	L.AddPrecacheMaterial(Texture'VehicleFX.Particles.DustyCloud2');
	L.AddPrecacheMaterial(Texture'XGameShadersB.Blood.BloodJetc');
	L.AddPrecacheMaterial(Texture'AW-2004Particles.Energy.SparkHead');
	L.AddPrecacheMaterial(Texture'XEffects.EmitSmoke_t');
	L.AddPrecacheMaterial(Texture'EmitterTextures.MultiFrame.rockchunks02');
	L.AddPrecacheMaterial(Texture'EmitterTextures.MultiFrame.MistTexture');
}

simulated function PostBeginPlay()
{
	SpawnEffects();
}

simulated function SpawnEffects()
{
	local Material SurfaceMat;
	local int HitSurface;
	local Vector HitLocation, HitNormal;
	local rotator EffectDir;
	local Actor Other;

	Other = Trace(HitLocation, HitNormal, Location + vector(Rotation) * 32, Location - vector(Rotation) * 16, true,, SurfaceMat);

	EffectDir = rotator(MirrorVectorByNormal(vector(Rotation), HitNormal));

	if(Vehicle(Other) != None && Other.SurfaceType == 0) //Hack to make vehicles behave like metal.
		HitSurface = 3;
	else if(Other != None && !Other.IsA('LevelInfo') && Other.SurfaceType != 0)
		HitSurface = Other.SurfaceType;
	else if(SurfaceMat != None)
		HitSurface = SurfaceMat.SurfaceType;
	else //No surface? Arghhh!!!!
	{
		Destroy();
		return;
	}

	if(PhysicsVolume.bWaterVolume)
		Spawn(class'pclImpactSmoke');
	else
		Spawn(HitEffectClasses[HitSurface],,,, EffectDir);

	if(Other != None && Other.bWorldGeometry && DecalClasses[HitSurface] != None)
		Spawn(DecalClasses[HitSurface]);
}

defaultproperties
{
	HitEffectClasses(0)=class'DirtHitEmitter' //EST_Default
	HitEffectClasses(1)=class'RockHitEmitter' //EST_Rock
	HitEffectClasses(2)=class'DirtHitEmitter' //EST_Dirt
	HitEffectClasses(3)=class'MetalHitEmitter' //EST_Metal
	HitEffectClasses(4)=class'WoodHitEmitter' //EST_Wood
	HitEffectClasses(5)=class'PlantHitEmitter' //EST_Plant
	HitEffectClasses(6)=class'FleshHitEmitter' //EST_Flesh
	HitEffectClasses(7)=class'SnowHitEmitter' //EST_Ice
	HitEffectClasses(8)=class'SnowHitEmitter' //EST_Snow
	HitEffectClasses(9)=class'WaterHitEmitter' //EST_Water
	HitEffectClasses(10)=class'GlassHitEmitter' //EST_Glass

	DecalClasses(0)=class'DefaultBulletDecal' //EST_Default
	DecalClasses(1)=class'DefaultBulletDecal' //EST_Rock
	DecalClasses(2)=class'SnowBulletDecal' //EST_Dirt
	DecalClasses(3)=class'MetalBulletDecal' //EST_Metal
	DecalClasses(4)=class'WoodBulletDecal' //EST_Wood
	DecalClasses(5)=None //EST_Plant
	DecalClasses(6)=None //EST_Flesh
	DecalClasses(7)=class'GlassBulletDecal' //EST_Ice
	DecalClasses(8)=class'SnowBulletDecal' //EST_Snow
	DecalClasses(9)=None //EST_Water
	DecalClasses(10)=class'GlassBulletDecal' //EST_Glass

	LifeSpan=5.0
	CullDistance=7000
	bHidden=true
}
